#!/usr/bin/python
# ZPhotoSlides, Zope product Copyright (C) 2002-2003  Jean-Francois Roche <jfroche@jfroche.be> Florent Carlier
# <florent.carlier@insa-rennes.fr>
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# $Id: CMFPhotoSlides.py,v 2.5 2004/10/06 19:07:40 jfroche Exp $'


from ZPhotoSlides import PhotoSlides
from ZPhotoSlidesBase import ZPhotoSlidesBaseItem
from ZPhotoSlidesUtils import have_stripogram,have_localizer,have_PIL,have_IM,tmp_folder
from permissions import cmfphotoslides_apply_security,exportPhoto
from Globals import InitializeClass
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent
from Products.CMFDefault.SkinnedFolder import SkinnedFolder
import AccessControl

factory_type_information = {
  'id'             : 'ZPhotoSlides',
  'portal_type'    : 'CMF ZPhotoSlides',
  'meta_type'      : 'CMF ZPhotoSlides',                         # This is NOT a ZPhotoSlides
  'description'    : 'A photo album',
  'content_icon'   : 'photo.gif',                                # Icon has to be acquired from a skin folder
  'product'        : 'ZPhotoSlides',
  'factory'        : 'addZPhotoSlides',
  'immediate_view' : 'zphotoslides_view',
  'filter_content_types' : 1,                                    # This is for folderish types and zps is NOT folderish
  'allowed_content_types': ('ZPhoto',),                          
  'actions': (
    {
    'id'           : 'folder_contents',
    'name'         : 'Contents',
    'action'       : 'zphotoslides_contents',
    'permissions'   : (ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'           : 'view',
    'name'         : 'View',
    'action'       : 'zphotoslides_view',
    'permissions'  : (View, ),
    'category'      : 'object',
    },
    {
    'id'           : 'slide_show',
    'name'         : 'Slide show',
    'action'       : 'slide_show#show',
    'permissions'   : (View, ),
    'category'      : 'object',
    },
    {
    'id'           : 'edit',                     
    'name'         : 'Edit',
    'action'       : 'folder_edit_form',        # use plone template. It's quite clean
    'permissions'   : (ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'           : 'properties',
    'name'         : 'Properties',
    'action'       : 'zphotoslides_edit_form',  # Our edit properties form without metadatas (id/description/etc.)
    'permissions'   : (ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'           : 'make_thumbs',
    'name'         : 'Make thumbnails',
    'action'       : 'make_thumbs',
    'permissions'   : (ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'           : 'add_photos',
    'name'         : 'Add Photos',
    'action'       : 'add_photos',
    'permissions'   : (ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'           : 'stats',
    'name'         : 'Statistics',
    'action'       : 'stats_zphotos',
    'permissions'   : (ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'           : 'manage_postcards',
    'name'         : 'Manage PostCards',
    'action'       : 'manage_postcards',
    'condition'    : 'python: object.show_postcard',
    'permissions'   : (ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'           : 'manage_ratings',
    'name'         : 'Manage Ratings',
    'action'       : 'manage_ratings',
    'condition'    : 'python: object.show_rating',
    'permissions'   : (ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'           : 'abouts',
    'name'         : 'About this product',
    'action'       : 'abouts',
    'permissions'   : (ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'           : 'export',
    'name'         : 'Export photo',
    'action'       : 'zphoto_export',
    'permissions'   : (exportPhoto, ),
    'category'      : 'object',
    },
    #{
    #'id'            : 'metadata',
    #'name'          : 'Metadata',
    #'action'        : 'metadata_edit_form',
    #'permissions'   : (ModifyPortalContent, ),
    #'category'      : 'object',
    #},
    {
    'id'            : 'folderlisting',
    'name'          : 'View',
    'action'        : 'zphotoslides_view',
    'permissions'   : (View, ),
    'category'      : 'folder',
    'visible'       : 0,
    },
    )
}


def addZPhotoSlides(self,id,title=''):
    ''' Add a CMF ZPhotoSlides '''
    z = ZPhotoSlides(id, title)
    self._setObject(id,z)
    
class CMFPhotoSlides(PhotoSlides,ZPhotoSlidesBaseItem): # Use SkinnedFolder instead of Folder. It's a CMF content
    ''' CMF ZPhotoSlides '''

    
    meta_type = 'CMF ZPhotoSlides'
    manage_options = ( # what management options are there?
	               {'label': 'Contents',       'action': 'manage_main'},
	               {'label': 'edit',       'action': 'zphotoslides_edit_form'},
                     )

    
    have_stripogram = have_stripogram
    have_localizer = have_localizer
    have_PIL = have_PIL
    have_IM = have_IM
    tmp_folder = tmp_folder

    security = AccessControl.ClassSecurityInfo()
    cmfphotoslides_apply_security(security,View,ModifyPortalContent)

    def __init__(self):
        PhotoSlides.__init__(self)
        ZPhotoSlidesBaseItem.__init__(self)
        
    #def folder_listing(self,REQUEST=None):
    #    ''' default view for folder '''
    #    if REQUEST:
    #        REQUEST.RESPONSE.redirect( '%s/zphotoslides_view' % self.absolute_url() )

    # We CANT overide the folder_contents method. This crash some templates !
    # so an authorised user is able to view the template via is url :(
    #def folder_contents(self,REQUEST=None):
    #    ''' contents view for folder '''
    #    if REQUEST:
    #        REQUEST.RESPONSE.redirect( '%s/zphotoslides_contents' % self.absolute_url() )

    def displayContentsTab(self):
        ''' we dont want folder content tabs for zphoto ! '''
        return None

    def SearchableText(self):
        ''' texte to index in catalog '''
        return '%s %s' % ( self.Title(), self.Description(), )

try:
    from Products.CMFPlone.PloneFolder import PloneFolder
    class ZPhotoSlides(CMFPhotoSlides,PloneFolder):
        ''' CMF ZPhotoSlides '''

        __implements__ = PloneFolder.__implements__
    
        security = AccessControl.ClassSecurityInfo()
        cmfphotoslides_apply_security(security,View,ModifyPortalContent)

        def __init__(self,id,title=''):
            try:
                PloneFolder.__init__(self,id=id,title=title)
                CMFPhotoSlides.__init__(self)
            except:
                PloneFolder.__dict__['__init__'](self,id=id,title=title) #__init__(self,id=id,title=title)
                CMFPhotoSlides._dict__['__init__'](self) #.__init__(self)
            
except:
    class ZPhotoSlides(CMFPhotoSlides,SkinnedFolder):
        ''' CMF ZPhotoSlides '''

        __implements__ = SkinnedFolder.__implements__

        security = AccessControl.ClassSecurityInfo()
        cmfphotoslides_apply_security(security,View,ModifyPortalContent)

        def __init__(self,id,title=''):
            try:
                SkinnedFolder.__init__(self,id=id,title=title)
                CMFPhotoSlides.__init__(self)
            except:
                SkinnedFolder.__dict__['__init__'](self,id=id,title=title) #__init__(self,id=id,title=title)
                CMFPhotoSlides.__dict__['__init__'](self) #__init__(self)
            
InitializeClass(ZPhotoSlides)
