#!/usr/bin/python
# ZPhotoSlides, Zope product
# Copyright (C) 2002-2003  Jean-Francois Roche <jfroche@jfroche.be>
#                          Florent Carlier <florent.carlier@insa-rennes.fr>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# $Id: ZPhotoSlides.py,v 2.25 2004/10/10 15:13:13 jfroche Exp $'
#

from Globals import HTMLFile      # fakes a method from a DTML file
from Globals import HTML
from Globals import package_home
from Globals import MessageDialog # provid
from Globals import InitializeClass
from OFS.Folder import Folder
from OFS.Image  import Image, cookId
import AccessControl
import traceback
import os, sys
import math
from math import floor
import time
from DateTime import DateTime
import random
import string
from cStringIO import StringIO
import shutil
import postalcard
import rating
import re, whrandom
from stat import stat
from stat import ip_log
from photo_edition.IMG_IM_start import *
from Products.MailHost import MailHost
from Products.PythonScripts import PythonScript
from Products.StandardCacheManagers.AcceleratedHTTPCacheManager import AcceleratedHTTPCacheManager
from ZPhotoSlidesBase import ZPhotoSlidesBaseItemWithComment,SkinnedItem
from ZPhotoSlidesUtils import *
from ZPSPloneBatch import Batch
from zipfile import ZipFile
try:
    import tarfile
except:
    have_tar = 0
else:
    have_tar = 1
from permissions import zphotoslides_apply_security
from config import *

if have_PIL:
    from photo_edition.IMG_start import *
    from photo_edition.image_modif import Image_modif

if have_localizer:
    from Products.Localizer import LocalDTMLFile
    from Products.Localizer import LocalPageTemplateFile
    from Products.Localizer import Gettext
    zpsgettext = Gettext.translation(globals())
else:
    from Globals import DTMLFile
    from Products.PageTemplates.PageTemplateFile import PageTemplateFile
    LocalDTMLFile = DTMLFile
    LocalPageTemplateFile = PageTemplateFile
    del DTMLFile
    del PageTemplateFile
    from ZPhotoSlidesUtils import zpsgettext

_ = zpsgettext


manage_addZPhotoSlidesForm = LocalDTMLFile('dtml/ZPhotoSlidesAdd',globals(),
                                                tmp_folder=find_tmp_folder(),
                                                pil=have_PIL,
                                                convert=have_IM,
                                                zpsconfig=zpsconfig()
                                          )


def manage_addZPhotoSlides(self,id,lib='PIL',convert='',manager_mail='',
                            double_name=1,remove=0,title='',show_postcard=0,show_rating=0,
                            photo_folder='',tmp_folder='',
                            allow_new_folder=0,recursive=0,description='',keywords='',
                            use_http_cache=None,
                            REQUEST=None):
    """Add a ZPhotoSlides to a folder."""
    # compatibility whith old template remove it later
    show_postcard = REQUEST.get('postcard',show_postcard)
    show_rating = REQUEST.get('ratings',show_rating)
    mailhost = REQUEST.get('mailhost',None)
    mailport = REQUEST.get('mailport',None)
    if allow_new_folder == 'on':allow_new_folder = 1
    if recursive == 'on':recursive = 1
    id = string.strip(id)
    if(id == ''):
        raise IOError,"You didn't enter a valid id! ('" + id + "') !!"
    
    if(lib=="IM"):
        if(convert=='' or (not os.path.isfile(convert))):
            if(have_IM != ''): # we use the found convert, user couldn't give another one cause hidden in form
                convert = have_IM
            else:
                raise IOError,"Can't find Image Magick CONVERT in '%s'!" % convert
        
    if  show_postcard and not re.search(r"^([\w_\.\-])+\@(([\w\-])+\.)+([\w]{2,4})+$", manager_mail):
	raise IOError,"You didn't enter a valid email!"
    tmpdir = tmp_folder
    
    self._setObject(id, ZPhotoSlides(id,title))
    ZPS = self._getOb(id)

    for item in ('show_postcard','show_rating','recursive','double_name','use_http_cache'):
        if eval(item) in (1,'1','on'): setattr(ZPS,item,1)
        else:setattr(ZPS,item,0)

    ZPS.manager_mail = manager_mail
    ZPS.show_ARpostcard = ZPS.show_postcard
    ZPS.manage_afterAdded(title,description,keywords,mailhost,mailport)

    ZPS.manage_addFS(photo_folder,tmpdir)

    ZPS.list_rating = rating.ratings()

    get_transaction().commit()

    result = msg = ''
    if photo_folder:
        result,msg = ZPS.makeThumbs(lib,convert,photo_folder,tmpdir,0,REQUEST,force=1,redirect=0)
    
    if REQUEST:
        result = 'ZPhotoSlides configure success %s ' % msg
        return REQUEST.RESPONSE.redirect( '%s/zphotoslides_edit_form?zps_message=%s' % (ZPS.absolute_url(),str(result)) )

class PhotoSlides:
    '''PhotoSlides Common class'''
    access=AccessControl.getSecurityManager()
 
    security = AccessControl.ClassSecurityInfo()

    isPrincipiaFolderish = 1
    
    updateVisited = updateVisited 
    parse_ext = parse_ext
    
    def __init__(self):
        """initialise a new instance of ZPhotoSlides"""
        self.description = ''                   # description of zphotoslides
        self.update_date = DATE                 # last update
        self.show_postcard = 0
        self.show_ARpostcard = 0
        self.show_rating = 1
        self.size = 0
        self.max_size = MAX_SIZE*1024
        self.sort_field = 'position'
        self.allow_export = EXPORT_ACCESS
        self.show_export = EXPORT_ACCESS
        self.visits_log = stat()
        self.non_hidden_pic = 0                 # amount of photos not hidden and visible for normal user
        self.list_non_hidden_pic = []           # list of the photos not hidden and visible for normal user
        self.rows = 3                           # default amount of row for layout in rendering the main table of photos
        self.column = 3                         # default amount of colum for layout in rendering the main table of photos
        self.zphoto_header= ''                  # get content of standard header
        self.list_photo = {}                    # list of all the photos
        self.zphoto_footer= ''                  # get content of standard footer
        self.symbolic_photo = 'random'          # name of the selected symbolic photo (default to random)
        self.keywords = ''                      # keywords describing the object
        self.description = ''                   # description of the object
        self.first_big = 0                      # default size for slide show
        self.show_automatic_slide_show = 1      # bool to print or not the slideshow
        self.show_viewed = 1                    # bool to print or not the amount of visitors for a photo
        self.show_exif = 1			# show the exif infos!
        self.photo_space = 30			# space between the photos in index_html
        self.last_modif = DateTime()
        self.show_iptc = 1
        self.formats_available = zpsconfig()['DEFAULT_SIZES']
        self.default_photo_size = DEFAULT_FORMAT
        self.formats = ['thumbs','medium']
        self.actual_css = 'stylesheet_default_css' # main stylesheet
        self.thumb_width = 200
        self.thumb_height = 200
        self.list_rating = rating.ratings()
        self.photo_folder = ''                  # empty folder for CMF
        self.tmpdir = find_tmp_folder()
        self.lib = get_img_lib()
        self.convert = find_convert()
        self._setProperty('amount_photo',0,'int')
        self._setProperty('use_http_cache',0,'int')
        self.use_http_cache = 0
        
    security.declarePrivate('manage_afterAdded')
    def manage_afterAdded(self,title='',description='',keywords='',mailhost='localhost',mailport=25):
        self.title = title
        self.description = description
        self.keywords = 'ZPhotoSlides,' + keywords
        if not self.portal_type:
            if not hasattr(self,'main_template'):
                self.manage_restoreSkin()
            self.manage_updateMailHost(mailhost=mailhost,mailport=mailport)
            # aq show_breadcrumbs from folder
            if hasattr(self.aq_parent,'show_breadcrumbs'):
                self.show_breadcrumbs = self.aq_parent.show_breadcrumbs
            else:
                self.show_breadcrumbs = 0
    def setsort(self):
        "set sor"
        self.sort_field = 'position'

    security.declarePrivate('manage_addFS')
    def manage_addFS(self,photo_folder=None,tmpdir=None):
        if not tmpdir:
            tmpdir = find_tmp_folder()
        if tmpdir:
            path = self._createDirectory(tmpdir)
            self.tmp_dir = path
        if photo_folder and ACCESS_FS == 1:
            self.photo_folder = photo_folder

                                                                                
    def update_cache(self):
        ''' Cache *Photo and DTML methods. 
        This is usefull if we use Squid (or a proxy cache) as Zope frontend.
        '''
        if 'ZPSHTTPCache' not in self.objectIds():
            self._setObject('ZPSHTTPCache',AcceleratedHTTPCacheManager('ZPSHTTPCache'))
        if self.use_http_cache:
            cache_types = ['CMF ZPhoto','ZPhoto','DTML Method']
        else:
            cache_types = ['DTML Method']

        # get obj in *current* folder
        objs = self.ZPSHTTPCache.ZCacheManager_locate(require_assoc=0, subfolders=0,meta_types=cache_types)
        dict = {}
        for elem in objs:
            res = "associate_%s" % elem['path']
            dict[res] = 1
        self.ZPSHTTPCache.ZCacheManager_setAssociations(props=dict)

    def getBatch(self, files, b_size, b_start, orphan):
      """ Patch ZPT """
      return Batch(files, b_size, int(b_start), orphan=0)
    
    def rate_content(self,id_photo=None,rating=None,REQUEST=None):
      " Rate Content "
      sessionID = self.generateId()
      comment = ''
      try:
        self.list_rating.add_new(sessionID,id_photo,rating,comment)
        status_msg = _('Rating has been calculated.')
      except:
        status_msg = _('Anonymous not allowed to rate content.')
      REQUEST.set('message_admin',status_msg)
      REQUEST.set('photoid',id_photo)
      return self.ratings_box(REQUEST=REQUEST,zpsgettext=zpsgettext,message_admin=status_msg,photoid=id_photo)

    def set_sort(self,sort,REQUEST):
        "Set the sorting rule"
        self.sort_field = sort
        message = 'Album is yet sorted on %s' % sort
        if REQUEST:
            return REQUEST.RESPONSE.redirect( '%s/zphotoslides_contents?zps_message=%s' % (self.absolute_url(),message) )

    
    def get_rating_all(self):
        "Print all ratings for a photo"
	liste_dictrate = []
        i = 0
	for elem in self.list_rating:
	            dictrate = {}
                    dictrate['sequence-number'] = i
		    dictrate['sessionID'] = elem.sessionID
		    dictrate['id_photo'] = elem.id_photo
		    dictrate['rating'] = elem.rating
		    dictrate['comment'] = elem.comment
		    liste_dictrate.append(dictrate)
                    i = i+1
	return liste_dictrate
	
    def get_rating_list(self,id_photo):
        "Print all ratings for a photo"
	liste_dictrate = []
        i = 0
	for elem in self.list_rating:
	    if(elem.id_photo==id_photo):
	            dictrate = {}
                    dictrate['sequence-number'] = i
		    dictrate['sessionID'] = elem.sessionID
		    dictrate['id_photo'] = elem.id_photo
		    dictrate['rating'] = elem.rating
		    dictrate['comment'] = elem.comment
		    liste_dictrate.append(dictrate)
                    i = i+1
	return liste_dictrate
    def getDefaultSize(self):
        "return the default size for a Photo"
        return self.default_photo_size

    def get_rating(self,id_photo):
	"Print the ratings for a Photo"
        liste_dict = {}                             # new list
	total = 0
        for elem in self.list_rating:               # for all ratings
            if(elem.id_photo==id_photo):                   #   if rating for the photo
	            total += 1
		    if (elem.rating not in liste_dict.keys()):
		       liste_dict[elem.rating] = 1
		    else:
		       liste_dict[elem.rating] = liste_dict[elem.rating] + 1
	for elem in liste_dict.keys():
	    liste_dict[elem] = (round((float(liste_dict[elem])/float(total))*100.0,2), liste_dict[elem])
        return liste_dict                               # we return the list
 
    def get_ratingfolder(self):
        "Print the ratings for Album"
	liste = {}
	total = len(self.list_rating)
	for elem in self.list_rating:
		if (elem.rating not in liste.keys()):
			liste[elem.rating] = 1
		else:
			liste[elem.rating] = liste[elem.rating] + 1
				
	for elem in liste.keys():
                liste[elem] = (round((float(liste[elem])/float(total))*100.0,2),liste[elem])
	return liste
    
    security.declarePrivate('generateId')
    def generateId(self):
	" Generate a unique, random id"
        s=string.lowercase + string.digits
        newid=''
        #suffix is 20 characters long
        for x in range(20):
        	newid=newid+whrandom.choice(s)
	return newid
	
    security.declarePrivate('validEmail')    
	
    security.declarePrivate('visited')
    def visited(self,REQUEST):
        iplog = ip_log(REQUEST)
        self.visits_log.new_visit(iplog,REQUEST)

    security.declarePublic('get_exif_infos')
    def get_exif_infos(self,photo_id):
        if(self.list_photo.has_key(photo_id + '_info')):
            res = self.list_photo[photo_id + '_info']
        else:
            res = {}
        return res

    security.declarePublic('get_iptc_infos')
    def get_iptc_infos(self,photo_id):
        if(self.list_photo.has_key(photo_id + '_iptc')):
           res = self.list_photo[photo_id + '_iptc']
        else:
           res = {}
        return res

    security.declarePrivate('getIPTCPropName')
    def getIPTCPropName(self,key):
        return key.replace("_"," ")

    def choose_symbolic_photo(self,ids=None,REQUEST=None):
	"Choose the symbolic photo for the ZPhotoSlides Folder"
        if ids is None: message = 'You must choose a photo!'
        if len(ids) > 1: message = 'You must choose only one photo!'
        if len(ids) == 1:
            photo_name=ids[0]
            self.symbolic_photo = photo_name
            self.last_modif = DateTime()
            message = 'Change of symbolic photo sucessful! Symbolic photo is now: %s' % photo_name
        if REQUEST:
            return REQUEST.RESPONSE.redirect( '%s/zphotoslides_contents?zps_message=%s' % (self.absolute_url(),message) )
    def showExport(self):
        ''' show the export tab '''
        return self.show_export

    def export(self,format,newwidth,newheight,REQUEST={}):
        """Export an album with all photos in defined format inside a zip file"""
        list = []
        sIO = StringIO()
        zip = ZipFile(sIO,"w",8)
        albumname = self.getId() + '.zip'
        for img in self.imagesValues():
            newname, file = img.export(format,newwidth,newheight,0)
            zip.writestr(newname,file)
        zip.close()
        if(REQUEST!={}):
            REQUEST.RESPONSE.setHeader('Content-Type','application/zip')
            REQUEST.RESPONSE.addHeader("Content-Disposition","filename=%s" % albumname)
            REQUEST.RESPONSE.write(sIO.getvalue())
        else:
            return albumname,sIO.getValue()
            
    def number_of_ratings(self,id_photo):
	"Returns the number of ratings"
	count = 0
	for elem in self.list_rating:
		if(elem.id_photo == id_photo):
			count = count + 1
	return count

    def getPostcards(self):
        ''' return all postcards '''
        liste = []
        for o in self.imagesValues():
            liste += o.getPostcards()
        return liste

    def delPostcards(self,ids,REQUEST=None):
        ''' delete postcards in ids '''
        if ids:
            for o in self.imagesValues():
                o.delPostcards(ids)
            message = 'Postcards have been deleted: %s' % ' '.join(ids)
        else:
            message = 'You must selected one or more postcards to delete.'
        if REQUEST:
            return REQUEST.RESPONSE.redirect( '%s/manage_postcards?zps_message=%s' % (self.absolute_url(),message) )

    def delete_rating(self,sessionID,REQUEST):
	"Delete a rating from a photo fallowing the comment date"
	for rate in self.list_rating:
		if(rate.sessionID==sessionID):
			self.list_rating.remove(rate)
			break
	if REQUEST is not None:
		return self.manage_rating(manage_tabs_message=_('Rating deleted for')+ str(sessionID))
		
    security.declarePrivate('get_keywords')
    def get_keywords(self,photo_id):
	string = ''
	for imag in self.imagesValues():
		if(imag.getId()==photo_id):
			cpt = 0
			for elem in imag.keywords:
			   if(cpt == 0):
				string = elem
			   else:
			   	string = string + ', ' + str(elem).strip()
			   cpt= cpt +1
			break
	return string		
    

    security.declarePublic('give_header_4_robots')
    def give_header_4_robots(self):
	"give the header for the robots"
	return """
<meta name="KEYWORDS" content="%s" />
<meta name="DESCRIPTION" content="%s" />
<meta name="resource-type" content="document" />
<meta name="ROBOTS" content="INDEX,FOLLOW" />
<meta http-equiv="expires" content="0" />
<meta name="author" content="ZPhotoSlides" />
<meta name="copyright" content="ZPhotoSlides %s" />
<meta name="revisit-after" content="1 days" />
<meta name="rating" content="General" />""" % (self.keywords,self.description,self.version)
    
    security.declarePrivate('getSlideShowContents')
    def getSlideShowContents(self,size='medium'):
        "get the list of photos for the Javascript of the slideshow"
        objects = self.imagesValues(1)
        list_string = 'var photo_count = %i;\n' % len(objects)
        cpt = 1
        for elem in objects:
            dict = {}
            tag = 'photo_tags[%d] = \'%s name="photo"/>\';\n' % ( cpt, elem.tag(size)[:-2] )
            url = 'photo_urls[%d] = "%s?size=%s";\n' % (cpt,elem.absolute_url(), size)
            title = 'photo_captions[%d] = "%s";\n' % (cpt,elem.title_or_id())
            list_string += tag + url + title 
            cpt = cpt + 1
        return list_string
	    
    security.declarePrivate('amount_photos')
    def amount_photos(self):
        return len(self.imagesValues(1))

    def get_size(self):
        '''return the size of the object, sum of the sizes of all photos'''
        size = 0
        for img in self.imagesValues():
            size = size + img.get_size()
        return size

    def imagesValues(self,suppressHiddenFiles=0):
        ''' return thumbs object '''
        if self.portal_type:
            return self.aq_explicit.listFolderContents()
        list = self.objectValues(['ZPhoto','CMF ZPhoto'])
        if suppressHiddenFiles:
            return [i for i in list if i.getviewit !='0']
        else:
            return list

    def imagesIds(self,suppressHiddenFiles=0):
        ''' return thumbs ids '''
        return [i.getId() for i in self.imagesValues(suppressHiddenFiles)]

    def countImages(self,suppressHiddenFiles=0):
        ''' return len of images objects '''
        return len( self.imagesValues(suppressHiddenFiles) )
    
    def getThumb(self,id,size='thumb'):
        ''' return an image '''
        if hasattr(self,id): return self[id]
        else: return None
        
    def getThumbTag(self,id,size='thumb'):
        ''' return an image tag '''
        return self.getThumb(id,size=size).tag(size)
    
    def getSymbolicTag(self):
        thumb = None
        if self.symbolic_photo == 'random':
            images = self.imagesValues(1)
            if len(images):
                thumb = whrandom.choice(self.imagesValues(1))
        else:
            thumb = self.getThumb(self.symbolic_photo)
        if thumb:
	    return thumb.tag('thumb')
        else:
            return '<img src="%s/misc_/ZPhotoSlides/zps_unknow.gif" width="100" heigth="100">' % self.absolute_url()

    security.declarePublic('getSlide')
    def getSlide(self,thumb_id):
	"""Give the number of the slide where is the photo"""
	thumb_pic = self.getThumb(thumb_id)
	pos= int(thumb_pic.position)
	tmp = (pos)/(self.rows*self.column)
	if(pos <= self.rows*self.column):
		tmp=0
	return tmp

    def manage_edit(self,id,title,keywords,description,photo_space,default_photo_size,rows,column,photo_folder='',manager_mail='',
                    show_breadcrumbs=0,show_exif=0,show_iptc=0,first_big=0,show_postcard=0,show_rating=0,
                    show_ARpostcard=0,show_automatic_slide_show=0,show_viewed=0,show_export=0,mailhost=None,REQUEST=None):
        """does this really need a doc string?"""
        self.id = id
        self.title = title
        self.default_photo_size = default_photo_size
        self.keywords = 'ZPhotoSlides,' + keywords
        self.description = description
        self.photo_space = int(photo_space)
        for item in ('show_breadcrumbs','show_exif','show_iptc','first_big','show_postcard','show_rating','show_ARpostcard','show_automatic_slide_show','show_viewed','show_export'):
            if eval(item) in ('1','on'): setattr(self,item,1)
            else: setattr(self,item,0)
        result = ''

        if not self.portal_type and REQUEST:
            self.manage_edit_comments(REQUEST)
            
        self.rows = int(rows)
        if(not self.validEmail(manager_mail)) and show_postcard:
            if REQUEST:
                message = 'You didn\'t enter a valid email!'
                return REQUEST.RESPONSE.redirect( '%s/zphotoslides_edit_form?zps_message=%s' % (self.absolute_url(),message) )
        else:
            if REQUEST:
                self.manager_mail = manager_mail
                mailhost = REQUEST.get('mailhost',None)
                mailport = REQUEST.get('mailport',None)
        if mailhost and self.show_postcard:
            self.manage_updateMailHost(mailhost,mailport)
        self.column = int(column)
        self.last_modif = DateTime()
        if REQUEST:
            message = 'Properties for %s changed. %s' % (self.id,result)
            return REQUEST.RESPONSE.redirect( '%s/zphotoslides_edit_form?zps_message=%s' % (self.absolute_url(),message) )

    def makeThumbs(self,lib=None,convert=None,photo_folder='',tmpdir='',use_http_cache=0,REQUEST=DEFAULT_SIZES,force=0,redirect=1):
        ''' Delete all thumbnails. And regenerate thumbs with an optional new fs dir.
        REQUEST       contain a dict {size_name:size, loc_name:zodb/fs, ..}
        photo_folder  fs dir
        force         if 1, force regenerate thumbs from fs
        '''
        elapse = time.time()
        c_elapse = time.clock()

        config = zpsconfig()
        
        message = 'Nothing to do.'
        need_thumbnailize = 0
        
        # check configuration
        if config['FORCE_IMAGE_LIBRARY']:
            self.lib = config['FORCE_IMAGE_LIBRARY']
        else:
            if lib: self.lib = lib

        if config['CONVERT_LOCATION']:
            self.convert = config['CONVERT_LOCATION']
        else:
            if convert: self.convert = convert 
                
        if config['TMP_DIR']:
            tmpdir = config['TMP_DIR']
                
        formats = self.formats_available.keys()
        #update formats size and location
        for item in ('size','location','used'):
            for f in formats:
                value = REQUEST.get('%s_%s' % (item,f), None)
                if value:
                    # if a format ('size','location','used') change, we have to rethumbnailize
                    if self.formats_available[f][item] != value:
                        need_thumbnailize = 1
                    self.formats_available[f][item] = value
                else:
                    if item == 'used' and f not in ('thumb','full'):
                        if self.formats_available[f][item] == 1:
                            need_thumbnailize = 1
                        self.formats_available[f][item] = 0
                    
        if photo_folder and (self.photo_folder != photo_folder or force):
            need_thumbnailize = 1
        
        if tmpdir and tmpdir != self.tmpdir and need_thumbnailize:
            self._createDirectory(tmpdir)
        
        # change fs dir
        if photo_folder and need_thumbnailize:
            if os.path.isdir(photo_folder):
                # test if we use fs to store thumbs (we can upload all size !)
                def test_formats(f):
                    if f['used'] == 1 and f['location'] == 'fs' and f['size'] != 'full':
                        return 1
                    else:
                        return None
                use_fs = len([v for v in self.formats_available.values() if test_formats(v)])
                if use_fs:
                    self._createDirectory(photo_folder)
                self.photo_folder = photo_folder
                
                # if we change fs dir then delete all thumbs
                ids = self.imagesIds()
                self.manage_delObjects(ids)
                
                # and recreate thumbcreencapture
                uploadOriginals(self,self.photo_folder)
                message = '%i images resized.' % len(self.imagesValues())
                #return (0,message)
            else:
                message = 'Error: the folder %s does not exist' % photo_folder
                if redirect:
                    return REQUEST.RESPONSE.redirect( '%s/make_thumbs?zps_message=%s' % (self.absolute_url(),message) )
                else:
                    return (0,message)
        else:
            # regenerate thumbs
            if need_thumbnailize:
                result = len([ph._pre_thumbnailize() for ph in self.imagesValues()])
                message = '%i images resized.' % result
        form = self.getSizes()
        if(self.default_photo_size not in form):
            self.default_photo_size = form[1]

        # update cache
        if use_http_cache: self.use_http_cache = 1
        else: self.use_http_cache = 0
        self.update_cache()

        message = '%s Time needed: %s s. CPU Time needed: %s s' % (message, time.time() - elapse, time.clock() - c_elapse)
        if redirect:
            return REQUEST.RESPONSE.redirect( '%s/make_thumbs?zps_message=%s' % (self.absolute_url(),message) )
        else:
            return ( len( self.imagesValues() ), message )
        
    security.declarePrivate('remove_thumbnails_from_disk')
    def remove_thumbnails_from_disk(self):
        #don't need this. It's a temp folder !
        return
        for img in self.tmp.getFile():
		tmpdir = os.path.join(self.tmp_dir,img)
		try:
			os.remove(tmpdir)
		except:
			pass # we dont want to delete folder

    def is_full(self):
        '''Check if album is full'''
        if(MAX_SIZE!=0 and self.size > (MAX_SIZE*1024)):
            return 1
        else:
            return 0

    def add_new_photo(self,REQUEST):
	"Add new photos"
        cpt = 0
        for elem in REQUEST.form.keys():
            file = REQUEST.form[elem]
            if(type(file)!=type('') and file.filename!=''):
                name,ext = os.path.splitext(file.filename)
                list = {}
                ext = string.lower(ext)
                if(ext == '.zip'):
                    zip = ZipFile(file,"r",8)
                    for filename in zip.namelist():
                        path,newfilename = os.path.split(filename)
                        data = zip.read(filename)
                        if(len(data)):
                            list[newfilename] = data
                elif(ext in ['.tar','.gz','.bz2'] and have_tar):
                    if(ext == '.tar'):
                        tar = tarfile.open(mode="r|",fileobj=file)
                    elif(ext == '.gz'):
                        tar = tarfile.open(mode="r|gz",fileobj=file)
                    elif(ext == '.bz2'):
                        tar = tarfile.open(mode="r|bz2",fileobj=file)
                    for filename in tar:
                        path,newfilename = os.path.split(filename.name)
                        if(filename.isfile()):
                            data = tar.extractfile(filename)
                            list[newfilename] = data.read()
                else:
                    id, title = cookId(None,None,file)
                    file.seek(0)
                    list[id] = file.read()
                for key in list:
                    if(MAX_SIZE!=0 and self.size > MAX_SIZE*1024):
                        del list[key] # dont count this photo as added!
                        break
                    if self.portal_type:
                        self.invokeFactory('ZPhoto',key,RESPONSE=None)
                        zphoto = self._getOb(key)
                        zphoto.formats_available['full']['location'] = 'zodb'
                        zphoto.manage_upload(StringIO(list[key]))
                        
                    else:
                        self.manage_addProduct['ZPhotoSlides'].manage_addZPhoto(key,'',StringIO(list[key]))
                    if(not self.__dict__.has_key('size')):
                        self.size = self.get_size()
                    self.size = self.size + len(list[key])
                cpt = cpt + len(list)
        self.update_cache()
        message = '%i Photos added!' % cpt
        return REQUEST.RESPONSE.redirect( '%s/add_photos?zps_message=%s' % (self.absolute_url(),message))

    def folder_delete(self,ids=None,remove=0,REQUEST=None):
        """Remove of the thumbnails/small/medium/large.
        
           WARNING: This MUST overide folder method in plone
        """
        if not ids:
            message = 'You must select one or more photos !'
        else:
            if self.portal_type:
                self.manage_delObjects(ids)
            else:
                for rfile in ids:
                    if rfile != 'random':
                        img = self.getThumb(rfile)
                        # Remove Thumbnails and Ajust position
                        pos_del = img.position
                        imgsize = img.get_size()
                        self.manage_delObjects(rfile)
                        for img in self.imagesValues():
                            if(img.position > pos_del):
                                img.position = img.position - 1
            self.update_cache()
            message = 'Image(s) have been deleted'
        if REQUEST:
            return REQUEST.RESPONSE.redirect( '%s/zphotoslides_contents?zps_message=%s' % (self.absolute_url(),message) )
			
    security.declarePrivate('back_from_form')
    def back_from_form(self,REQUEST):
	text = ''
	for elem in REQUEST.form.keys():
		value = REQUEST.form[elem]
		if(value==''):
		   value='0'
		if(elem!='file'):
		   text = text + '<input type="hidden" name="%s" value="%s">' % (elem,value)
	return text
	
    security.declarePublic('view_preview')
    def view_preview(self,REQUEST):
	"""View the preview of an image"""
	img = self.image_modif(REQUEST,1)
	image = REQUEST.file
        # this is obsolete, have to replace or dont check. tmpdir is always there !
	#self.check_tmp_folder() 
	location = os.path.join(self.tmp_dir,img.imgname)
	img.save_to(location)
	hidden_part = self.back_from_form(REQUEST)
        self.tmp.update_file_list()
	msg2 = ''
	if(hasattr(REQUEST,'show_both') and REQUEST.show_both=='on'):
		msg2 = _('<table><tr><td class="boxborder"><center><h4>Before modification:</h4></center><br>') + '<img src="%s/img/%s?%d"></td>' % (REQUEST.URL1+'',image,random.randint(0,100))
                both = 1
        else:
                msg2 = ''
                both = 0
        if(self.portal_type):
                return REQUEST.RESPONSE.redirect('%s/preview_modif?img_src=%s&file=%s&both=%i' %
                (self.absolute_url(),img.imgname,REQUEST.file,both))
        else:
	        return MessageDialog2(
		        title = _('Preview modification of') + image,
			message1 = _('<td class="boxborder"><center><h4>After modification: </h4></center><br><center>') + '<img src="%s/tmp/%s?%d"></center></td></tr></table>' % (REQUEST.URL1+'',img.imgname,random.randint(0,100)),
			message2 = msg2,
			hidden_field = hidden_part,
	  		action = 'edit_image',
			id = 'file',
			id_content = image,
			URL1 = REQUEST.URL1,
			actual_css = self.actual_css
			)

    def getZPhoto(self,id):
        for elem in self.imagesValues():
            if(id == elem.getId()):
                return elem
        return None

    def doTransform(self,ids,REQUEST):
        '''Transform the selected photos'''
        for imgid in ids:
            img = self.getZPhoto(imgid)
            img.doTransform(REQUEST,0,0)
        return REQUEST.RESPONSE.redirect('%s/zphotoslides_contents?zps_message=Modifications done !' % self.absolute_url())

    def batchAskTransform(self,REQUEST):
        """Beginning of batch transform"""
        if(not REQUEST.has_key('ids') or len(REQUEST.ids)==0):
            return REQUEST.RESPONSE.redirect('%s/zphotoslides_contents?zps_message=You must choose at least one photo!' % self.absolute_url())
        list = []
        for elem in REQUEST.ids:
            list.append(self.getZPhoto(elem))
        REQUEST.set('ids',list)
        return self.zphoto_edit_form(REQUEST)

    def get_image_info(self,image_name):
        """Get all the informations about an image"""
        path = self.image_folder.basepath
        abspath = os.path.join(path,image_name)
        img = Image_modif(abspath)
        dict = img.get_info()
        return dict

    def undoable_transactions(self, first_transaction=None,
                              last_transaction=None,
                              PrincipiaUndoBatchSize=None):
        pass

    def undo_modif(self,REQUEST):
	"""Undo a previous modification of an image"""
	image = REQUEST.file
	bak = image + '.bak'
	path = self.image_folder.basepath
	new = os.path.join(path,image)
	backup = os.path.join(path,bak)
	if os.path.exists(backup):
	    shutil.copyfile(backup,new)
	    os.remove(backup)
            if(self.portal_type):
                return REQUEST.RESPONSE.redirect('%s/edit_photo?file=%s&portal_status_message=%s' %
                                    (self.absolute_url(),REQUEST.file,'Undo successfull !'))
            else:
	        return self.edit_image(file=image,manage_tabs_message=_("Undo successfull !"))
	else:
            if(self.portal_type):
                return REQUEST.RESPONSE.redirect('%s/edit_photo?file=%s&portal_status_message=%s' %
                                    (self.absolute_url(),REQUEST.file,"Sorry, undo can't be done for this image."))
            else:
	        return self.edit_image(file=image,manage_tabs_message=_("Sorry, undo can't be done for this image."))
 
    security.declarePublic('viewable_images')               
    def viewable_images(self):
        ''' return list of non hidden photos '''
        self.list_non_hidden_pic = self.imagesValues(1)
        self.non_hidden_pic = len(self.list_non_hidden_pic)
	self.list_non_hidden_pic.sort(lambda x, y: cmp(x.position, y.position))
	return self.list_non_hidden_pic
    get_nbr_non_hidden_thumb = viewable_images
    
    security.declarePublic('encodemail')               
    def encodemail(self, emailvalue, namevalue):
        """ Encodage des emails """
        hs='0123456789ABCDEF'
        mystring='document.write(\'<a href=\"mailto:'+emailvalue+'\"><b>'+namevalue+'</b></a>\')'
        newstring=''

        length=len(mystring)
        for i in range(length):
           newstring=newstring+'%'+hs[int(floor((ord(mystring[i]))/16))]+hs[(ord(mystring[i]))%16]
      
        return '<script language=\"JavaScript\">eval(unescape(\''+newstring+'\'))</script>'

    def get_ordered_visits(self):
        "Return the list of most visited photos in this album"
        items = self.imagesValues()
        items.sort(lambda x,y: -cmp(x.viewed,y.viewed))
        return items[:10]

    def visits_this_month(self):
        return self.visits_log.this_month()

    def visits_today(self):
        items = self.visits_log.today()
        items.sort(lambda x,y: -cmp(x.visit_amount,y.visit_amount))
        return items

    def total_visits(self):
        return self.visits_log.total_visits()

class ZPhotoSlides(PhotoSlides,ZPhotoSlidesBaseItemWithComment,SkinnedItem,Folder):
    ''' ZPhotoSlides 1.4 '''

    meta_type = 'ZPhotoSlides' # what do people think they're adding?
    portal_type = None
    isPrincipiaFolderish = 1

    #fake member
    visible_ids = 1
    
    if(have_localizer):
    	locale = package_home(globals()) + '/locale/'
    	_zpsgettext = zpsgettext
    	

    manage_options = (
	               {'label': 'Contents',       'action': 'manage_main'},
	               {'label': "Properties", 'action': 'zphotoslides_edit_form'},
		       {'label': "View",       'action': 'index_html'}, # defaults to index_html
		       {'label': "Edit skin", 'action': 'manage_skin'},
                       {'label': "List all files & Edit Photo", 'action': 'zphotoslides_contents'},
		       {'label': "Add photos", 'action':'add_photos'},
                       {'label': "Make thumbnails", 'action': 'make_thumbs'},
		       {'label': "Statistics",'action':'manage_stats'},
		       {'label': "Manage PostCards",'action': 'manage_postcards'},
		       {'label': "Manage Ratings",'action': 'manage_ratings'},
	               {'label': "Security",   'action': 'manage_access'},
		       {'label': "About this product",   'action': 'abouts'}
		     )

    _main_template = LocalPageTemplateFile('zpt/main_template',globals())
    zphotoslides_edit_form = LocalPageTemplateFile('skins/zphotoslides/zphotoslides_edit_form',globals())
    zphotoslides_header = LocalPageTemplateFile('skins/zphotoslides/zphotoslides_header',globals())
    zps_batch_macros = LocalPageTemplateFile('skins/zphotoslides/zps_batch_macros',globals())
    edit_photo = LocalPageTemplateFile('skins/zphotoslides/edit_photo',globals())
    zphotoslides_contents = LocalPageTemplateFile('skins/zphotoslides/zphotoslides_contents',globals())
    make_thumbs = LocalPageTemplateFile('skins/zphotoslides/make_thumbs',globals())
    add_photos = LocalPageTemplateFile('skins/zphotoslides/add_photos',globals())
    index_html = LocalPageTemplateFile('skins/zphotoslides/zphotoslides_view',globals())
    #view for compatibility
    view = LocalPageTemplateFile('skins/zphotoslides/zphotoslides_view',globals())
    preview_modif = LocalPageTemplateFile('skins/zphotoslides/preview_modif',globals())
    view_photo = LocalPageTemplateFile('skins/zphotoslides/view_photo',globals()) 
    view_movie = LocalPageTemplateFile('skins/zphotoslides/view_movie',globals()) 
    slide_show = LocalPageTemplateFile('skins/zphotoslides/slide_show',globals())
    slide_show_js = LocalDTMLFile('skins/zphotoslides/slide_show_js',globals(),zpsgettext=zpsgettext)
    abouts = LocalPageTemplateFile('skins/zphotoslides/abouts',globals())
    ratings_box = LocalPageTemplateFile('skins/zphotoslides/ratings_box',globals())
    ratings_view = LocalPageTemplateFile('skins/zphotoslides/ratings_view',globals())
    manage_ratings = LocalPageTemplateFile('skins/zphotoslides/manage_ratings',globals())
    view_card =  LocalPageTemplateFile('skins/zphotoslides/view_card',globals())
    manage_stats = LocalPageTemplateFile('skins/zphotoslides/zphotoslides_stats',globals())
    manage_postcards = LocalPageTemplateFile('skins/zphotoslides/manage_postcards',globals())
    zphoto_edit_form = LocalPageTemplateFile('skins/zphotoslides/zphoto_edit_form',globals()) 
    zphotoslides_export = LocalPageTemplateFile('skins/zphotoslides/zphoto_export',globals())

    # old dtml
    edit_image = LocalDTMLFile('dtml/edit_photo',globals(),zpsgettext=zpsgettext)
    manage_tabs = LocalDTMLFile('dtml/manage_tabs',globals(),zpsgettext=zpsgettext)
    view_iptc = LocalDTMLFile('dtml/view_iptc',globals(),zpsgettext=zpsgettext)


    security = AccessControl.ClassSecurityInfo()
    zphotoslides_apply_security(security,'View','View management screens')
        
    def __init__(self,id,title=''):
        try:
            PhotoSlides.__init__(self)
            SkinnedItem.__init__(self)
            ZPhotoSlidesBaseItemWithComment.__init__(self)
            Folder.__init__(self,id=id)
        except:
            PhotoSlides.__dict__['__init__'](self) #__init__(self)
            SkinnedItem.__dict__['__init__'](self) #__init__(self)
            ZPhotoSlidesBaseItemWithComment.__dict__['__init__'](self) #__init__(self)
            self.id = id

    def getIcon(self):
        ''' return icon location '''
        return '%s/misc_/ZPhotoSlides/photo.gif' % self.absolute_url()
    
    def manage_updateMailHost(self,mailhost='localhost',mailport=25):
        ''' update/create a MailHost object '''
        if self.show_postcard==1:
            self.mailhost = mailhost
            self.mailport = mailport
            try:
                self.zmailhost.manage_makeChanges('ZPSMailHost',mailhost,mailport,None)
            except:
                self.manage_addProduct['MailHost'].manage_addMailHost('zmailhost', 'ZPSMailHost',mailhost,mailport)
        else:
            self.mailhost = ''
            self.mailport = ''

def check_bad_id(id):
    oldid = id
    id = re.sub(r'[^a-zA-Z0-9-_~,.$\(\)#]','',id)
    return oldid,id

def uploadOriginals(self,path):
    prefix = [i['name'] for i in self.getFormats()]
    dir = path
    list = os.listdir(dir)
    for elem in list:
        if(MAX_SIZE !=0 and self.get_size()>MAX_SIZE*1024): # check max size (defined in config.py)
            break
        pref = elem[:elem.find('_')]
        if pref not in prefix:
            img_dir = os.path.join(dir,elem)
            if elem[-4:].lower() in ('.jpg','.gif','.png'):
                oldid,elem = check_bad_id(elem)
                if self.portal_type:
                    self.invokeFactory('ZPhoto',elem,RESPONSE=None)
                    self[elem].update_data_from_fs(img_dir)
                else:
                    self.manage_addProduct['ZPhotoSlides'].manage_addZPhoto(elem,elem,path=img_dir)
                photo = self[elem]
                photo.filename = oldid
                photo.dir = dir
                photo.path = img_dir
                photo._pre_thumbnailize()
                self.size = self.get_size() + photo.get_size() # increase size

InitializeClass(ZPhotoSlides)
