# ZPhotoSlides, Zope product
# Copyright (C) 2002-2003  Jean-Francois Roche <jfroche@jfroche.be>
#                          Florent Carlier <florent.carlier@insa-rennes.fr>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# $Id: image_modif.py,v 2.6 2004/10/10 15:13:14 jfroche Exp $'

import os, sys, string
import PIL.Image
from PIL import ImageFont
from PIL import ImageDraw
from PIL import ImageOps
from PIL import ImageChops
from PIL import ImageEnhance
from PIL import ImageFilter
import traceback
import shutil

default_textpos = 'b' # t=top, b=bottom;  
default_textalign = 'l' # l=left, c=center, r=right;
textcolor_default = 0xff0000

class Image_modif:
    def __init__(self, data):
        ## PIL Fonts : http://effbot.org/pil/pilfonts.zip ##
#                self.fullpath = path
#       fullpath = os.path.abspath(os.path.normpath(path))
#                self.imgpath = fullpath
        try:
                self.im = PIL.Image.open(data)
        except:
                print traceback.print_exc()
                raise 'LibraryError','PIL error while opening the photo'

#       self.imgname = os.path.basename(self.imgpath)
#       self.basepath = os.path.dirname(self.imgpath)
        self.format = self.im.format         # JPEG, GIF, ...
        self.imgmode = self.im.mode
#       self.rotname = self.imgname

    def make_backup(self):
        outputdir = self.basepath
        self.default = os.path.join(self.basepath,self.rotname)
        backup = self.default + '.bak'
        try: # make a backup of the old photo before rotation
            shutil.copyfile(self.default,backup)
        except IOError, e:
            return "Error on file %s : %s" % (self.imgpath,e)
    def save_to(self,location):
        try: # make a backup of the old photo before rotation
            self.im.save(location)
        except:
            return "eh merde"
    def get_info(self):
        format = self.im.format
        mode = self.im.mode
        size = self.im.size
        info = self.im.info
        dict = {'format': format,'mode': mode,'height':size[1],'width':size[0]}
        return dict

    def save_modif(self):
        try:
            self.im.save(self.default)
        except:
            traceback.print_exc()
            raise IOError,"Error on file %s!" % self.fullpath
    def getrotate(self,angle):
        """rotate a photo with a given angle"""
        angle = int(angle)
        self.im = self.im.rotate(angle)

    def gettextcopyright(self, textcopy, textpos, textalign, width, height, textfont, textsize, textboldfont=0, textitalicfont=0, textcolor='#FF0000'):
        """Florent Carlier V0.1 - copyright a photo with a given text

	        boldfont:   0 ... normal font (the default)
                	    1 ... bold font

	        textpos:    The position of the copyright text can be specified by a string
        	            containing a combination of one characters.  One character
                	    must be either t, b or m.

		textalign:  The Alignemnt of the copyright text can be specified by a string
        	            containing a combination of one characters.  One character
                	    must be either l, c or r.

                    t ... top
                    b ... bottom
		    m ... middle

                    l ... left
                    c ... center
                    r ... right

                    The default value is 'bc'

        """
        img = self.im
        textfont = string.replace(textfont, "S", textsize)
        if textboldfont == '1':
            if textitalicfont == '1':
                boldfont = string.replace(textfont, "T", 'BI')
            else:
                boldfont = string.replace(textfont, "T", 'B')
            font = ImageFont.load_path(boldfont)
        elif textitalicfont == '1':
            italicfont = string.replace(textfont, "T", 'I')
            font = ImageFont.load_path(italicfont)
        else:
            normalfont = string.replace(textfont, "T", 'R')
            font = ImageFont.load_path(normalfont)
        tw, th = font.getsize(textcopy)
        tw -= 2
        th -= 2
        if 't' in textpos:
            y = -1
        elif 'm' in textpos:
            y = (height/2) - th - 2
        else: # The default value is 'b'
            y = height - th - 2
        if 'l' in textalign:
            x = 2
        elif 'r' in textalign:
            x = (width - tw) - 2
        else: # The default value is 'c'
            x = (width - tw) / 2
        draw = ImageDraw.Draw(img)
        textcol = self.hex_to_rbg(textcolor)
        draw.text((x,y), text=textcopy, font=font, fill=textcol)
        del draw

    def change_Brightness(self,brightness):
        """Change the Brightness of an image  0.0 -> 1.0"""
        self.im = ImageEnhance.Brightness(self.im).enhance(brightness)

    def change_contrast(self,contrast):
        """Change the contrast of an image  0.0 -> 1.0"""
        self.im = ImageEnhance.Contrast(self.im).enhance(contrast)

    def auto_contrast(self):
        """Normalize image contrast"""
        self.im =  ImageOps.autocontrast(self.im)

    def equalize(self):
        """Equalize the image histogram."""
        self.im = ImageOps.equalize(self.im)

    def add_borders(self,border=0,fill='#FFFFFF'):
        """Add border pixels of border to the image fill color is HTML hex"""
        self.im = ImageOps.expand(self.im, border, self.hex_to_rbg(fill))

    def crop_image(self,border):
        """Remove border from image. The same amount of pixels are removed from all four sides."""
        self.im = ImageOps.crop(self.im,border)

    def change_Sharpness(self,sharpness):
        """Change the Sharpness of an image  0.0 -> 2.0 """
        self.im = ImageEnhance.Sharpness(self.im).enhance(sharpness)

    def change_Color(self,color):
        """Change the Color of an image  0.0 -> 1.0 """
        self.im = ImageEnhance.Color(self.im).enhance(color)

    def filter_blur(self):
        """Filter blur """
        self.im = self.im.filter(ImageFilter.BLUR)

    def filter_contour(self):
        """Filter contour """
        self.im = self.im.filter(ImageFilter.CONTOUR)

    def filter_detail(self):
        """Filter detail """
        self.im = self.im.filter(ImageFilter.DETAIL)

    def filter_enhance(self):
        """Filter enhance """
        self.im = self.im.filter(ImageFilter.EDGE_ENHANCE)
    def filter_emboss(self):
        """Filter emboss """
        self.im = self.im.filter(ImageFilter.EMBOSS)
    def filter_find_edges(self):
        """Filter find_edges """
        self.im = self.im.filter(ImageFilter.FIND_EDGES)
    def filter_smooth(self):
        """Filter smooth """
        self.im = self.im.filter(ImageFilter.SMOOTH)
    def filter_sharpen(self):
        """Filter sharpen """
        self.im = self.im.filter(ImageFilter.SHARPEN)
    def transpose_left_right(self):
        """Transpose left to right"""
        self.im = ImageOps.mirror(self.im)
    def invert_image(self):
        """Invert an image"""
        self.im = ImageChops.invert(self.im)
    def solarize(self,threshold):
        self.im = ImageOps.solarize(self.im, threshold=128)

	#	, DETAIL, EDGE_ENHANCE, EDGE_ENHANCE_MORE, EMBOSS, FIND_EDGES,
	#SMOOTH, SMOOTH_MORE, and SHARPEN.

    def hex_to_rbg(self,string):
        a = int(string[1],16)
        b = int(string[2],16)
        c = int(string[3],16)
        d = int(string[4],16)
        e = int(string[5],16)
        f = int(string[6],16)
        x = (a * 16) + b;
        y = (c * 16) + d;
        z = (e * 16) + f;
        tuple = (x,y,z)
        return tuple

    def resize_image(self,height,width,ratio):
        assert height!=0.0,  'Height is 0! Impossible!'
        assert width!=0.0, 'Width is 0! Impossible!'	
        photo_size = (width,height)
        self.im = self.im.resize(photo_size)
        return self.im

    def remove_color(self):
        self.im = self.im.convert('L')

    def location_of_fonts(self):
        location = os.path.dirname(globals()['__file__'])
        location = location + os.sep + 'fonts'
        return location

    def modify_from_request(self,img,REQUEST):
        modif = 0
        if(hasattr(REQUEST,'copyright') and REQUEST.copyright=='on'):
            if(os.path.isdir(self.location_of_fonts())):
                sys.path.append(self.location_of_fonts())
                (oldwidth,oldheight) = self.im.size
                #oldwidth = int(REQUEST.old_width)
                #oldheight = int(REQUEST.old_height)
                img.gettextcopyright(REQUEST.textcopy, REQUEST.position, REQUEST.align, oldwidth, oldheight, REQUEST.textfont, REQUEST.textsize, REQUEST.textboldfont, REQUEST.textitalicfont, REQUEST.textcolor)
            modif = 1
        if(hasattr(REQUEST,'rotate') and REQUEST.rotate=='on'):
                img.getrotate(REQUEST.angle)
                modif = 1
        if(hasattr(REQUEST,'grayscale') and REQUEST.grayscale=='on'):
                img.remove_color()
                modif = 1
        if(hasattr(REQUEST,'change_size') and REQUEST.change_size=='on'):
                modif = 1
                width = float(REQUEST.width)
                height = float(REQUEST.height)
                if(hasattr(REQUEST,'keep_ratio') and REQUEST.keep_ratio=='on'):
                        oldheight = float(REQUEST.old_height)
                        oldwidth = float(REQUEST.old_width)
                        ratio = oldwidth/oldheight
                        if(oldheight!=height and oldwidth==width): # height is modified
                            width = ratio * height
                        elif(oldheight==height and oldwidth!=width): # width is modified
                            height = width/ratio
                        elif(oldheight!=height and oldwidth!=width):
                            width = ratio * height
                        img.resize_image(int(height),int(width),ratio)
                else:
                        img.resize_image(height,width,0)
        if(hasattr(REQUEST,'change_color') and REQUEST.change_color=='on'):
                modif = 1
                img.change_Color(int(REQUEST.color)/100)

        if(hasattr(REQUEST,'change_brightness') and REQUEST.change_brightness=='on'):
                modif = 1
                img.change_Brightness(int(REQUEST.brightness)/100.0)

        if(hasattr(REQUEST,'change_contrast') and REQUEST.change_contrast=='on'):
                modif = 1
                img.change_contrast(int(REQUEST.contrast)/100.0)

        if(hasattr(REQUEST,'change_sharpness') and REQUEST.change_sharpness=='on'):
                modif = 1
                img.change_Sharpness(int(REQUEST.sharpness)/100.0)

        if(hasattr(REQUEST,'filter_blur') and REQUEST.filter_blur=='on'):
                modif = 1
                img.filter_blur()

        if(hasattr(REQUEST,'filter_contour') and REQUEST.filter_contour=='on'):
                modif = 1
                img.filter_contour()

        if(hasattr(REQUEST,'filter_detail') and REQUEST.filter_detail=='on'):
                modif = 1
                img.filter_detail()

        if(hasattr(REQUEST,'filter_enhance') and REQUEST.filter_enhance=='on'):
                modif = 1
                img.filter_enhance()

        if(hasattr(REQUEST,'filter_emboss') and REQUEST.filter_emboss=='on'):
                modif = 1
                img.filter_emboss()
        if(hasattr(REQUEST,'filter_find_edges') and REQUEST.filter_find_edges=='on'):
                modif = 1
                img.filter_find_edges()

        if(hasattr(REQUEST,'filter_smooth') and REQUEST.filter_smooth=='on'):
                modif = 1
                img.filter_smooth()

        if(hasattr(REQUEST,'filter_sharpen') and REQUEST.filter_sharpen=='on'):
                modif = 1
                img.filter_sharpen()

        if(hasattr(REQUEST,'equalize') and REQUEST.equalize=='on'):
                modif = 1
                img.equalize()

        if(hasattr(REQUEST,'auto_contrast') and REQUEST.auto_contrast=='on'):
                modif = 1
                img.auto_contrast()

        if(hasattr(REQUEST,'add_border') and REQUEST.add_border=='on'):
                modif = 1
                borderadd = int(REQUEST.border_to_add)
                fill_border = REQUEST.fill
                img.add_borders(borderadd,fill_border)

        if(hasattr(REQUEST,'crop_image') and REQUEST.crop_image=='on'):
                modif = 1
                border = int(REQUEST.remove_border)
                img.crop_image(border)

        if(hasattr(REQUEST,'invert_image') and REQUEST.invert_image=='on'):
                modif = 1
                img.invert_image()

        if(hasattr(REQUEST,'transpose_left_right') and REQUEST.transpose_left_right=='on'):
                modif = 1
                img.transpose_left_right()
        if(hasattr(REQUEST,'solarize') and REQUEST.solarize=='on'):
                modif = 1
                img.solarize(REQUEST.threshold)
        return (img, modif)


if __name__ == "__main__":
        a = Image_modif('/var/www/photos/marie03052003/MVC-008F.JPG')
        a.save_modif()
#a.get_info()
#a.make_backup()
#print hex()
#a.hex_to_int('#FFFFFF')
#a.resize_image(200,500)
#a.remove_color()
#a.auto_contrast()
#a.save_modif()
