# ZPhotoSlides, Zope product
# Copyright (C) 2002-2003  Jean-Francois Roche <jfroche@jfroche.be>
#                          Florent Carlier <florent.carlier@insa-rennes.fr>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# $Id: stat.py,v 2.1 2004/07/16 12:13:21 jfroche Exp $'
from OFS.SimpleItem import SimpleItem
import time
import socket
import string
import os
from DateTime import DateTime

def ip_log(REQUEST):
	ip_addr = ''
	if REQUEST.has_key('HTTP_X_FORWARDED_FOR'):
	    if REQUEST['HTTP_X_FORWARDED_FOR'] != '':
		ip_addr = REQUEST['HTTP_X_FORWARDED_FOR']
	if ip_addr == '' and REQUEST.has_key('REMOTE_ADDR'):
	    ip_addr = REQUEST['REMOTE_ADDR']
	if ip_addr == '':
	    ip_addr = 'None'
	return ip_addr


class visitor(SimpleItem):
	"a visitor item"
	def __init__(self,ip,REQUEST,visit_num):
		self.date = DateTime(time.time())
		self.ip = ip
		self.visit_amount = visit_num
		self.hostname = ''
		try:
			self.hostname = socket.gethostbyaddr(ip)[0]
		except:
			pass
		self.referer = ''
		self.navigator = ''
		if(string.find(REQUEST.HTTP_REFERER,REQUEST.BASE0)==-1):
			self.referer = REQUEST.HTTP_REFERER
		self.navigator = REQUEST.HTTP_USER_AGENT

	def __repr__(self):
		return str(self.date) + str(self.ip) + str(self.hostname) + str(self.referer) + str(self.navigator)
		

class stat(SimpleItem):
	"the statistic for visits of our objects"
	def __init__(self):
		self.visits = {}
		self.country = self.domain_country()
	def new_visit(self,ip,REQUEST):
		if(ip in self.visits.keys()):
			dict = visitor(ip,REQUEST,self.visits[ip].visit_amount + 1)
		else:
			dict = visitor(ip,REQUEST,1)
		l = self.visits
		visit = self.visits
		visit[ip] = dict
		self.visits = visit
	def __len__(self):
		return self.total_visits()
	def total_visits(self):
		sum = 0
		for elem in self.visits.keys():
			sum = sum + self.visits[elem].visit_amount
		return sum
	def today(self):
                '''return amount of visit of the day'''
		list = []
		for elem in self.visits.keys():
			if(self.visits[elem].date.isCurrentDay()):
				list.append(self.visits[elem])
		return list
	
	def this_month(self):
		liste = []
		for elem in self.visits.keys():
			if(self.visits[elem].date.isCurrentMonth()):
				liste.append(self.visits[elem])
		return liste
	def referer_stat(self):
		list = []
		for elem in self.visits.keys():
			if(self.visits[elem].referer != ''):
				list.append(string.lower(self.visits[elem].referer))
		return list

	def get_navigator(self,navigator):
		list = ['Opera','MSIE','Lynx','Googlebot','Scooter','Mozilla']
		for elem in list:
			if(navigator.find(elem)!=-1):	
				return elem
		return 'Others'

	def navigator_stat(self):
		list = {}
		cpt = 0
		for elem in self.visits.keys():
			cpt = cpt + self.visits[elem].visit_amount
			if(self.visits[elem].navigator not in list.keys()):
				 list[self.get_navigator(self.visits[elem].navigator)] = self.visits[elem].visit_amount
			else:
				list[self.visit[elem].navigator] = list[self.visit[elem].navigator] + self.visits[elem].visit_amount
		for elem in list.keys():
                        list[elem] = (round((float(list[elem])/float(cpt))*100.0,2),list[elem])
		return list

	def country_source(self):
		country_list = {}
		cpt = 0
		for elem in self.visits.keys():
			if(self.visits[elem].hostname != ''):
				cpt = cpt + self.visits[elem].visit_amount
				tmp = self.visits[elem].hostname.split('.')
				tmp[-1] = tmp[-1].upper()
				if(tmp[-1] not in country_list.keys()):
					country_list[tmp[-1]] = (1 * self.visits[elem].visit_amount)
				else:
					country_list[tmp[-1]] = country_list[tmp[-1]] + (self.visits[elem].visit_amount)
		for elem in country_list.keys():
			country_list[elem] = (round((float(country_list[elem])/float(cpt))*100.0,2),country_list[elem])
		return country_list

	def get_ext_name(self,name):
		if(name in self.country.keys()):
			return self.country[name]
		else:
			return name

	def domain_country(self):
                location = os.path.dirname(globals()['__file__'])
		fp = open(os.path.join(location,'country-codes.txt'))
		lines = fp.readlines()
		dict = {}
		for elem in lines:
			ext,name = elem.split('   ')
			dict[ext] = name[:-1]
		return dict
